package zero1hd.rhythmbullet.audio.analyzer;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.util.MusicManager;

public class AudioAnalyzer implements Disposable {
	private ExecutorService exec;
	
	private SpectralFluxAnalysisRunnable sfar;
	private ThresholdCalcRunnable tcr;
	private PruneFluxRunnable pfr;
	private PeakDetectionRunnable pdr;
	
	public AudioAnalyzer(MusicManager musicManager) {
		exec = Executors.newSingleThreadExecutor();
		sfar = new SpectralFluxAnalysisRunnable(musicManager);
		tcr = new ThresholdCalcRunnable(sfar);
		pfr = new PruneFluxRunnable(tcr);
		pdr = new PeakDetectionRunnable(pfr, sfar.getPUID());
		
	}
	
	public void start() {
		exec.submit(sfar);
		exec.submit(tcr);
		exec.submit(pfr);
		exec.submit(pdr);
	}
	
	private void stop() {
		sfar.work = false;
		tcr.work = false;
		pfr.work = false;
		pdr.work = false;
	}

	@Override
	public void dispose() {
		stop();
		exec.shutdown();
	}
	
	public boolean isDone() {
		if ((sfar.isDone() && tcr.isDone() && pfr.isDone() && pdr.isDone())) {
			dispose();
			return true;
		}
		return false;
	}
}
