package zero1hd.rhythmbullet.audio.analyzer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.rhythmbullet.util.MusicManager;

public class PruneFluxRunnable implements Runnable {
	boolean work = true;
	private boolean done;
	private FloatArray bassSpectralFlux;
	private FloatArray mSpectralFlux;
	private FloatArray umSpectralFlux;
	private FloatArray bassThreshold, mThreshold, umThreshold;
	private FloatArray bassPrunned = new FloatArray();
	private FloatArray mPrunned = new FloatArray();
	private FloatArray umPrunned = new FloatArray();
	private MusicManager musicManager;
	public PruneFluxRunnable(ThresholdCalcRunnable tcr) {
		bassSpectralFlux = tcr.getBassSpectralFlux();
		mSpectralFlux = tcr.getmSpectralFlux();
		umSpectralFlux = tcr.getUmSpectralFlux();
		
		bassThreshold = tcr.getBassThreshold();
		umThreshold = tcr.getUmThreshold();
		mThreshold = tcr.getmThreshold();
		
		this.musicManager = tcr.getMusicManager();
	}
	
	@Override
	public void run() {
		//pruning data
		float prunnedCurrentVal;

		for (int i = 0; i < umSpectralFlux.size && work; i++) {
			prunnedCurrentVal = bassSpectralFlux.get(i) - bassThreshold.get(i);
			if (prunnedCurrentVal >= 0) {
				bassPrunned.add(prunnedCurrentVal);
			} else {
				bassPrunned.add(0);
			}
			
			prunnedCurrentVal = mSpectralFlux.get(i) - mThreshold.get(i);
			if (prunnedCurrentVal >= 0 ) {
				mPrunned.add(prunnedCurrentVal);
			} else {
				mPrunned.add(0);
			}
			
			prunnedCurrentVal = umSpectralFlux.get(i) - umThreshold.get(i);
			if (prunnedCurrentVal >= 0 ) {
				umPrunned.add(prunnedCurrentVal);
			} else {
				umPrunned.add(0);
			}
		}
		done = true;
		Gdx.app.debug("Audio Analyzer", "Data prunned.");
	}
	
	public FloatArray getBassSpectralFlux() {
		return bassSpectralFlux;
	}
	public FloatArray getmSpectralFlux() {
		return mSpectralFlux;
	}
	public FloatArray getBassThreshold() {
		return bassThreshold;
	}
	public FloatArray getmThreshold() {
		return mThreshold;
	}
	public FloatArray getUmSpectralFlux() {
		return umSpectralFlux;
	}
	public FloatArray getUmThreshold() {
		return umThreshold;
	}
	public FloatArray getBassPrunned() {
		return bassPrunned;
	}
	public FloatArray getmPrunned() {
		return mPrunned;
	}
	public FloatArray getUmPrunned() {
		return umPrunned;
	}
	public boolean isDone() {
		return done;
	}
	
	public MusicManager getMusicManager() {
		return musicManager;
	}
}
