package zero1hd.rhythmbullet.desktop.audio;

import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;
import com.badlogic.gdx.files.FileHandle;

import zero1hd.rhythmbullet.audio.wavedecoder.WAVSampleReader;
import zero1hd.rhythmbullet.util.MusicManager;

public class WAVManager implements MusicManager {
	private int readWindowSize = 1024;
	private int playbackIndex, readIndex;
	private Music music;
	private FileHandle fileHandle;
	private AudioInputStream ais;
	private AudioFormat af;
	private WAVSampleReader d;
	
	public WAVManager(FileHandle file) {
		this.fileHandle = file;
		try {
			ais = AudioSystem.getAudioInputStream(file.file());
			d = new WAVSampleReader(ais);
			af = ais.getFormat();
		} catch (UnsupportedAudioFileException | IOException e) {
			e.printStackTrace();
		}
		music = Gdx.audio.newMusic(file);
	}
	
	@Override
	public void dispose() {
		music.dispose();
		try {
			ais.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public int playbackIndexUpdate() {
		playbackIndex = (int) ((getSampleRate() * getDuration())/getReadWindowSize());
		return playbackIndex;
	}

	@Override
	public int getPlaybackIndexPosition() {
		return playbackIndex;
	}

	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public int readSampleFrames(float[] samples) {
		try {
			readIndex++;
			return d.readSamplesAsFrames(samples);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return 0;
	}

	@Override
	public long getSampleCount() {
		return ais.getFrameLength()*d.getChannels();
	}

	@Override
	public float getDuration() {
		return ais.getFrameLength()/af.getFrameRate();
	}

	@Override
	public float getSampleRate() {
		return af.getSampleRate();
	}

	@Override
	public void pause() {
		music.pause();
	}

	@Override
	public void play() {
		music.play();
	}

	@Override
	public boolean isPlaying() {
		return music.isPlaying();
	}

	@Override
	public float getPositionInSeconds() {
		return music.getPosition();
	}

	@Override
	public void setPosition(float position) {
		music.setPosition(position);
	}

	@Override
	public void setOnCompletionListener(OnCompletionListener listener) {
		music.setOnCompletionListener(listener);
	}

	@Override
	public void setVolume(float percent) {
		music.setVolume(percent);
	}

	@Override
	public boolean isFinishedLoading() {
		return true;
	}

	@Override
	public String getBasicSongName() {
		return fileHandle.nameWithoutExtension();
	}

	@Override
	public FileHandle getMusicFile() {
		return fileHandle;
	}

	@Override
	public int framesRead() {
		return readIndex;
	}
	
	@Override
	public int getChannelCount() {
		return d.getChannels();
	}

	@Override
	public Music getMusic() {
		return music;
	}
}
