package zero1hd.rhythmbullet.desktop.graphics.ui.pages;

import java.util.Observable;
import java.util.Observer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.MusicInfo;
import zero1hd.rhythmbullet.desktop.audio.MusicInfoController;
import zero1hd.rhythmbullet.desktop.audio.MusicListController;
import zero1hd.rhythmbullet.desktop.graphics.ui.components.MusicSelectable;
import zero1hd.rhythmbullet.desktop.graphics.ui.components.ScrollText;
import zero1hd.rhythmbullet.util.MusicManager;

public class MusicSelectionPage extends Page implements Observer {
	Preferences musicFileAnnotation;
	
	private MusicListController mc;
	private MusicInfoController mic;
	private Array<MusicSelectable> selectables;
	private Table musicTable;
	private ScrollPane musicTableScrollPane;
	private TextButton back;
	
	private Table musicInfoTable;
	private Table musicSubInfo;
	private ScrollText songTitle;
	private Label author;
	private Label songLength;
	private Label previousTop;
	private Label ratedDifficulty;
	private Texture albumCoverTexture;
	private Image albumCover;
	
	private AssetManager assets;
	private MusicSelectable currentlySelected;
	
	private Skin skin;
	private boolean down, up;
	private int musicSelectableIndex;
	
	private TextButton beginButton;
	
	private int uiSongCount;
	private int uiSongInfoCount;
	private float scrollTimer, scrollDelay = 0.2f, scrollDelMod, songSelectionTimer;
	
	public MusicSelectionPage(Skin skin, MusicListController musicListController, MusicInfoController musicInfoController, AssetManager assetManager, final Vector3 cameraTarget, final AnalysisPage ap) {
		setTextureBackground(assetManager.get("gradients.atlas", TextureAtlas.class).findRegion("round-grad"));
		setBgColor(Color.toFloatBits(110,164,193, 255));
		this.assets = assetManager;
		this.skin = skin;
		this.mc = musicListController;
		this.mic = musicInfoController;
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		musicTable = new Table();
		musicTableScrollPane = new ScrollPane(musicTable, skin);
		musicTable.defaults().spaceTop(5f).spaceBottom(5f);
		musicTableScrollPane.setSize(0.45f*getWidth(), getHeight());
		musicTableScrollPane.setFadeScrollBars(false);
		musicTableScrollPane.setOverscroll(false, false);
		addActor(musicTableScrollPane);
		selectables = new Array<>();
		back = new TextButton("Back", skin);
		back.setWidth(back.getWidth()+20f);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeight() - back.getHeight() - 15f);
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				cameraTarget.x = 0.5f*Gdx.graphics.getWidth();
			}
		});
		addActor(back);
		back.toFront();

		addListener(new InputListener() {
			@Override
			public boolean keyDown(InputEvent event, int keycode) {
				scrollTimer = 0;
				scrollDelMod = 1f;
				if (keycode == Keys.DOWN) {
					down = true;
				}
				if (keycode == Keys.UP) {
					up = true;
				}
				return super.keyDown(event, keycode);
			}
			
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.DOWN) {
					down = false;
				}
				
				if (keycode == Keys.UP) {
					up = false;
				}
				return super.keyUp(event, keycode);
			}
		});
		
		musicInfoTable = new Table();
		musicInfoTable.defaults().center();
		musicInfoTable.setPosition(musicTableScrollPane.getWidth() + musicTableScrollPane.getX(), 0);
		musicInfoTable.setSize(getWidth()-musicTableScrollPane.getWidth(), getHeight());
		addActor(musicInfoTable);
		musicSubInfo = new Table(skin);
		musicSubInfo.setBackground("corner-panel");
		songTitle = new ScrollText("", null, skin, true, true);
		author = new Label(null, skin, "sub-font", skin.getColor("default"));
		songLength = new Label(null, skin, "sub-font", skin.getColor("default"));
		previousTop = new Label(null, skin, "sub-font", skin.getColor("default"));
		ratedDifficulty = new Label(null, skin, "sub-font", skin.getColor("default"));
		albumCover = new Image(assetManager.get("defaultCover.png", Texture.class));
		
		beginButton = new TextButton("Begin", skin);
		beginButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				cameraTarget.x = 2.5f*getWidth();
				ap.processSong(mc.getMusicList().getAudioData(getSelectedMusic()), (albumCoverTexture != null ? albumCoverTexture : assets.get("defaultCover.png", Texture.class)), mic.getInfo(getSelectedMusic()));
			}
		});
	}
	
	@Override
	public void act(float delta) {
		if (down) {
			if (scrollDelMod > 0.25f) {
				scrollDelMod -= delta/0.5f;
			}
			if (scrollTimer <= 0) {
				scrollTimer = scrollDelay*scrollDelMod;
				scrollDown();

			} else {
				scrollTimer -= delta;
			}
		}
		
		if (up) {
			if (scrollDelMod > 0.25f) {
				scrollDelMod -= delta/0.5f;
			}
			if (scrollTimer <= 0) {
				scrollTimer = scrollDelay*scrollDelMod;
				scrollUp();
			} else {
				scrollTimer -= delta;
			}
		}
		
		if (songSelectionTimer > 0f) {
			songSelectionTimer -= delta;
			if (songSelectionTimer <= 0f) {
				setCurrentMusic();
			}
		}
		
		attemptRefreshUpdate();
		super.act(delta);
	}
	
	private void attemptRefreshUpdate() {
		if (uiSongInfoCount != mc.getMusicList().getTotal()) {
			if (uiSongCount < mc.getMusicList().getTotal()) {
				MusicSelectable selectable = new MusicSelectable(mc.getMusicList().getSongFileHandleFromIndex(uiSongCount), skin, this);
				selectables.add(selectable);
				musicTable.add(selectables.get(uiSongCount)).expandX().fillX();
				uiSongCount++;
				musicTable.row();
				if (uiSongCount == mc.getMusicList().getTotal()) {
					selectMusicUI(mc.getCurrentMusicManager());
				}
			} else if (uiSongInfoCount < selectables.size && mic.isDoneLoading()) {
				selectables.get(uiSongInfoCount).updateInfo(mic.getSongInfoArray().get(uiSongInfoCount));
				uiSongInfoCount++;
				if (uiSongInfoCount == selectables.size) {
					updateInformation();
				}
			}
		}
	}
	
	public FileHandle getSelectedMusic() {
		return currentlySelected.getMusicFile();
	}
	
	public MusicInfo getSelectedMusicInfo() {
		return currentlySelected.getMusicInfo();
	}
	
	public void refreshUIList() {
		for (int i = 0; i < selectables.size; i++) {
			selectables.get(i).dispose();
		}
		mic.loadSongInfo();
		musicTable.clear();
		selectables.clear();
		musicInfoTable.clear();
		musicSubInfo.clear();
		uiSongCount = 0;
		uiSongInfoCount = 0;
		
		Gdx.app.debug("MusicSelectionPage", "Refreshing...");
		
		musicInfoTable.add(songTitle).width(musicInfoTable.getWidth()*0.6f).spaceBottom(30f);
		musicInfoTable.row();
		musicSubInfo.add(author);
		musicSubInfo.row();
		musicSubInfo.add(songLength);
		musicSubInfo.row();
		musicSubInfo.add(previousTop);
		musicSubInfo.row();
		musicSubInfo.add(ratedDifficulty);
		musicSubInfo.pack();
		musicInfoTable.add(musicSubInfo).spaceBottom(20f);
		musicInfoTable.row();
		musicInfoTable.add(albumCover).size(musicInfoTable.getWidth()/2f);
		musicInfoTable.row();
		musicInfoTable.add(beginButton).spaceTop(20f).fillX();
	}
	
	@Override
	public void dispose() {
		super.dispose();
	}
	
	public void deselectAll() {
		if (currentlySelected != null) {
			currentlySelected.deselect();
		} else {
			for (int i = 0; i < selectables.size; i++) {
				selectables.get(i).deselect();
			}
		}

	}
	
	public void setCurrentlySelected(MusicSelectable currentlySelected) {
		this.currentlySelected = currentlySelected;
		songSelectionTimer = 1f;
		
		if (mic.isDoneLoading() && uiSongInfoCount == selectables.size) {
			updateInformation();
		}
	}
	
	private void scrollDown() {
		if ((musicSelectableIndex = (musicTable.getChildren().indexOf(currentlySelected, true) + 1)) == musicTable.getChildren().size) {
			musicSelectableIndex = 0;
		}
		((MusicSelectable)musicTable.getChildren().get(musicSelectableIndex)).select();
		musicTableScrollPane.scrollTo(currentlySelected.getX(), currentlySelected.getY(), currentlySelected.getWidth(), currentlySelected.getHeight());
	}
	
	private void scrollUp() {
		if ((musicSelectableIndex = (musicTable.getChildren().indexOf(currentlySelected, true) - 1)) < 0) {
			musicSelectableIndex = musicTable.getChildren().size-1;
		}
		((MusicSelectable)musicTable.getChildren().get(musicSelectableIndex)).select();
		musicTableScrollPane.scrollTo(currentlySelected.getX(), currentlySelected.getY(), currentlySelected.getWidth(), currentlySelected.getHeight());
	}

	@Override
	public void update(Observable o, Object arg) {
		if (o == mc) {
			MusicManager mm = (MusicManager) arg;
			selectMusicUI(mm);
		}
	}
	
	public void selectMusicUI(MusicManager mm) {
		if (currentlySelected == null || mm.getMusicFile() != currentlySelected.getMusicFile()) {
			for (int i = 0; i < selectables.size; i++) {
				if (selectables.get(i).getMusicFile() == mm.getMusicFile()) {
					selectables.get(i).select();
					musicSelectableIndex = i;
					break;	
				}
			}
		}
	}
	
	private void setCurrentMusic() {
		playSelectedMusic();
	}
	
	private void playSelectedMusic() {
		if (currentlySelected.getMusicFile() != mc.getCurrentMusicManager().getMusicFile()) {
			int index = mc.getMusicList().getMusicList().indexOf(currentlySelected.getMusicFile(), true);
			mc.setMusicByIndex(index);
		}
	}
	
	/**
	 * This should only be called when everything is loaded.
	 */
	private void updateInformation() {
		Gdx.app.debug("MusicSelectionPage", "Updating song info panel...");
		if (currentlySelected == null) throw new NullPointerException("Buddy, you need to update this page to have the proper current music selected...");
		if (currentlySelected.getMusicInfo() == null) throw new NullPointerException("K, ur music info is null.");
		songTitle.setText(currentlySelected.getMusicInfo().getMusicName(), null);
		author.setText("Author: " + currentlySelected.getMusicInfo().getAuthor());
		if (albumCoverTexture != null) {
			albumCoverTexture.dispose();
		}
		long lengthInSeconds = currentlySelected.getMusicInfo().getDurationInSeconds();
		int min = (int) (lengthInSeconds/60);
		int sec = (int) (lengthInSeconds - (min*60));
		songLength.setText("Length: " + min + ":" + (sec > 9 ? sec : "0" + sec));
		
		previousTop.setText("Highscore: " + currentlySelected.getMusicInfo().getPreviousTop());
		
		String difficulty = (getSelectedMusicInfo().getRatedDifficulty() == -1 ? "N/A" : String.valueOf(getSelectedMusicInfo().getRatedDifficulty()));
		ratedDifficulty.setText("Rated Difficulty: " + difficulty);
		albumCoverTexture = currentlySelected.getMusicInfo().loadTexture();
		if (albumCoverTexture != null) {
			albumCover.setDrawable((new TextureRegionDrawable(new TextureRegion(albumCoverTexture))));
		} else {
			albumCover.setDrawable((new TextureRegionDrawable(new TextureRegion(assets.get("defaultCover.png", Texture.class)))));
		}
	}
}
