package zero1hd.rhythmbullet.desktop.graphics.ui.pages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ProgressBar;
import com.badlogic.gdx.scenes.scene2d.ui.Slider;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.desktop.audio.MusicListController;

public class OptionsPage extends Page {
	Table optionsTable;
	private ProgressBar musicVolSlider;
	private ProgressBar fxVolSlider;
	private TextField directoryField;
	
	private float musicSearchTimer;
	public OptionsPage(final RhythmBullet core, final Vector3 targetPosition, KeybindOptionsPage moreOptionsPage, final MusicListController sc) {
		super("General", core.getDefaultSkin());
		setTextureBackground(core.getAssetManager().get("gradients.atlas", TextureAtlas.class).findRegion("round-grad"));
		setBgColor(Color.toFloatBits(93,75,144, 255));
		//Back button
		TextButton backButton = new TextButton("Back", core.getDefaultSkin());
		backButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.x = 0.5f*Gdx.graphics.getWidth();
			}
		});
		backButton.setPosition(10, getHeightBelowTitle() + 5);
		addActor(backButton);
		backButton.setWidth(backButton.getWidth() + 20);
		addSpaceToTitle(backButton.getWidth() + backButton.getX() + 20);
		
		optionsTable = new Table();
		optionsTable.defaults().center().space(10f);
		optionsTable.setSize(getWidth(), getHeight());
		addActor(optionsTable);
		
		Label musicVolSliderLabel = new Label("Music Volume: ", core.getDefaultSkin());
		optionsTable.add(musicVolSliderLabel);
		musicVolSlider = new Slider(0, 100, 0.1f, false, core.getDefaultSkin());
		musicVolSlider.setValue(core.getPrefs().getFloat("music vol", 100f)*100f);
		optionsTable.add(musicVolSlider).minWidth(0.3f*getWidth());
		final Label musicVolPercentage = new Label(MathUtils.round(musicVolSlider.getValue()) + "%", core.getDefaultSkin());
		musicVolSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				musicVolPercentage.setText(MathUtils.round(musicVolSlider.getValue()) + "%");
				sc.getCurrentMusicManager().setVolume(musicVolSlider.getPercent());
				
				core.getPrefs().putFloat("music vol", musicVolSlider.getPercent());
			}
		});
		optionsTable.add(musicVolPercentage);
		
		optionsTable.row();
		
		Label fxVolSliderLabel = new Label("FX Volume: ", core.getDefaultSkin());
		optionsTable.add(fxVolSliderLabel);
		fxVolSlider = new Slider(0, 100, 0.1f, false, core.getDefaultSkin());
		fxVolSlider.setValue(core.getPrefs().getFloat("fx vol", 100f)*100f);
		optionsTable.add(fxVolSlider).fillX();
		final Label fxVolPercentage = new Label(MathUtils.round(fxVolSlider.getValue()) + "%", core.getDefaultSkin());
		fxVolSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				fxVolPercentage.setText(MathUtils.round(fxVolSlider.getValue()) + "%");
				core.getPrefs().putFloat("fx vol", fxVolSlider.getPercent());
			}
		});
		
		optionsTable.add(fxVolPercentage);
		
		optionsTable.row();
		
		Label musicDirectoryLabel = new Label("Music Directory: ", core.getDefaultSkin());
		optionsTable.add(musicDirectoryLabel);
		final Label songCount = new Label("Songs: " + sc.getMusicList().getTotal(), core.getDefaultSkin(), "sub-font", core.getDefaultSkin().getColor("default"));
		directoryField = new TextField(null, core.getDefaultSkin() ) {
			@Override
			public void act(float delta) {
				if (musicSearchTimer > 0) {
					musicSearchTimer -= delta;
					if (musicSearchTimer <= 0) {
						sc.getMusicList().setSearchPath(directoryField.getText());
						sc.getMusicList().asynchRefresh();
						songCount.setText("Songs: " + sc.getMusicList().getTotal());
					}
				}
				super.act(delta);
			}
		};
		directoryField.setText(core.getPrefs().getString("music dir", System.getProperty("user.home")+System.getProperty("file.separator")+"Music"));
		directoryField.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				musicSearchTimer = 2;
				songCount.setText("...");
			}
		});
		optionsTable.add(directoryField).fillX();
		optionsTable.add(songCount);
		
		optionsTable.row();
		
		TextButton keybindSettings = new TextButton("Set Controls",  core.getDefaultSkin());
		keybindSettings.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.y = -0.5f*Gdx.graphics.getHeight();
			}
		});
		optionsTable.add(keybindSettings).colspan(2).fillX();
		
		optionsTable.row();
		
		TextButton graphicsSettings = new TextButton("Graphics",  core.getDefaultSkin());
		graphicsSettings.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.y = 1.5f*Gdx.graphics.getHeight();					
			}
		});
		optionsTable.add(graphicsSettings).colspan(2).fillX();
		
		optionsTable.row();
		
		Label fpsLabel = new Label("", core.getDefaultSkin()) {
			@Override
			public void act(float delta) {
				setText("Current Frames Per Second: " + Gdx.graphics.getFramesPerSecond());
				super.act(delta);
			}
		};
		optionsTable.add(fpsLabel).colspan(2);
		
		optionsTable.row();
		
		Label usageLabel = new Label("Current usage (lower the better): " + 100f*((float)(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())/(float)Runtime.getRuntime().totalMemory()) + "%", core.getDefaultSkin()) {
			float refreshTime = 60;
			@Override
			public void act(float delta) {
				refreshTime -= delta;
				if (refreshTime <= 0) {
					refreshTime = 60;
					setText("Current usage (lower the better): " + 100f*((float)(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())/(float)Runtime.getRuntime().totalMemory()) + "%");
				}
				super.act(delta);
			}
		};
		
		optionsTable.add(usageLabel).colspan(2);
	}
	
	public void saveOptions(Preferences prefs) {
		prefs.putString("music dir", directoryField.getText());
		Gdx.app.debug("Preferences", "Saved all basic options page values.");
	}
}
